//------------------------------------------------------------->
// ReportUnusedLocals.vw
//      R.Worsley 3/09/2008
//      Locate any unused local variables
//      Conditions:
//          Variables declared in the method line are not checked
//          Read notes below on StringTokenizer
//          Will work with the old "Local" as a descriptor
//          Variables can be declared anywhere in the method
//------------------------------------------------------------->
Use Windows.pkg
Use DFClient.pkg
Use File_dlg.pkg
Use cRegistry.pkg

Object oRegistry is a cRegistry
    Function GetReadTheKey Returns String
        Boolean bOpened
        String sKey sValue sPath

        Move "SOFTWARE\Data Access Worldwide\Visual DataFlex" to sKey
        Get OpenKey sKey to bOpened
        If bOpened Begin
            Move "UnusedLocals" to sValue
            If (ValueExists(Self, sValue)) Begin
                Get ReadString sValue to sPath
            End
            Send CloseKey // must close all open Keys
        End
        Function_Return sPath
    End_Function
    
    Procedure DoWriteTheKey String sPath
        String sKey sRegValue
        Boolean bOpened
        Integer iError
        Move "SOFTWARE\Data Access Worldwide\Visual DataFlex" to sKey
        Get OpenKey of oRegistry sKey to bOpened
        If bOpened Begin
            Move "UnusedLocals" to sRegValue
            If (ValueExists(oRegistry(Self), sRegValue));
                Send WriteString of oRegistry sRegValue sPath
            Else Begin
                Get CreateKey of oRegistry sKey to iError
                If (iError =0) Send WriteString of oRegistry sRegValue sPath
            End
            Send CloseKey to oRegistry
        End
    End_Procedure

End_Object


// vdf-Guidance.com
Use vWin32fh.pkg

// AUTHOR: Torkild Resheim
// Original 1.0 version modified slightly to work
// better when using a space as a token.  Presence
// of two spaces between variables would result 
// in a blank "not found" variable.
// Original version will work fine with this caution.
Use StringTokenizer.pkg

Object oTokenizer is a StringTokenizer
    Set Separator to " "
End_Object

Enum_List
    Define VarName
    Define VarStatus
    Define MethodName
    Define Line#
End_Enum_List

Function sGetSourceDirectory Global Returns String
    Handle hoWorkspace
    String sPath
    Integer iPos
    Get phoWorkspace of oApplication to hoWorkspace
    Get psAppSrcPath of hoWorkspace to sPath
    Get vFolderFormat sPath to sPath

    Get GetReadTheKey of oRegistry to sPath

    If (sPath = "") Begin 
        // In case of multiple paths appended in config.sys
        Move (pos(";", sPath)) to iPos  
        If (iPos > 0) Begin
            Move (Left(sPath, (iPos-1))) to sPath
            Get vFolderFormat sPath to sPath
        End
    End
    Function_Return sPath   
End_Function

// Simply removes the first word of any input string
// all up to the first space encountered
Function sGetRemoveFirstWord Global String sString Returns String
    Integer iPos
    Move (trim(sString)) to sString
    Move (Pos(" ", sString)) to iPos
    Function_Return (Right(sString, (Length(sString)-iPos))) 
End_Function

// Parses out the name of any method from the rest of the line
Function sGetMethodName Global String sMethod Returns String
    Integer iPos
    Get sGetRemoveFirstWord sMethod to sMethod
    Move (Right(sMethod, (Length(sMethod)-iPos))) to sMethod
    If (uppercase(Left(trim(sMethod), 4)) = "SET ");
        Get sGetRemoveFirstWord sMethod to sMethod
    // Remove any trailing arguments if any
    Move (Pos(" ", sMethod)) to iPos
    Function_Return (left(sMethod, (iPos-1)))    
End_Function

Deferred_View Activate_oReportUnusedLocals for ;
Object oReportUnusedLocals is a dbView

    Set Border_Style to Border_Thick
    Set Size to 138 300
    Set Location to 2 2
    Set Label to "Search for Unused Locals"
    Set piMinSize to 138 300

    Procedure DoParseLocals
        String sPath sFileName sLine sVars sSingleVar sMethodName sTest sOrigMethodName
        Integer iPos iArrayCount iMax iCounter iItems iLineCount iFound iOutputType
        String[][4] sLocalVars

        Get Value of oSourceName to sFileName 
    	Direct_Input channel 1 sFileName
    	If (SeqEof) Begin
            Send Stop_Box "Program not found!"
            Abort
    	End

        Move (ParseFolderName(sFileName)) to sPath
        Send DoWriteTheKey to oRegistry sPath
    	
        Get Current_Radio of oOutputRadioGroup to iOutputType
        If (iOutputType = 1) Begin 
//            Get sGetSourceDirectory to sPath
            Direct_Output channel 2 (sPath+"UnusedLocals.log")
            Writeln channel 2 "Line # - Method Name - Variable Name"
            Writeln channel 2 "------------------------------------"
        End
        Else Begin 
            Showln "Line # - Method Name - Variable Name"
            Showln "------------------------------------------------------"
        End

        Move (ResizeArray(sLocalVars, 0)) to sLocalVars 
    
    	Repeat
    	    Readln channel 1 sLine
    	    Increment iLineCount
    	    // Don't bother with comments
    	    If (Left(trim(sLine), 2) <> "//") Begin 
    	        Move (uppercase(trim(sLine))) to sMethodName
                If (Left(sMethodName, 10) = "PROCEDURE " or left(sMethodName, 9) = "FUNCTION ") Begin
                    Get sGetMethodName sLine to sOrigMethodName  // Looks nice for the report
        	        Repeat
                        //------------------------------------------------->
            	        Readln channel 1 sLine      // go to the next line
    	                Increment iLineCount
                        // Parse the variables
            	        If (uppercase(left(trim(sLine), 5)) = "LOCAL") Get sGetRemoveFirstWord sLine to sLine
                        Move (Uppercase(trim(sLine))) to sTest
            	        // Remove "local" if there
                        If (left(sTest, 6) = "STRING" or; 
                            left(sTest, 7) = "INTEGER" or; 
                            left(sTest, 6) = "NUMBER" or;
                            left(sTest, 4) = "DATE" or;
                            left(sTest, 6) = "HANDLE" or;
                            left(sTest, 7) = "BOOLEAN") Begin
            	            Move (trim(sLine)) to sLine
                            // Remove the data type command
            	            Move (pos(" ", sLine)) to iPos   // first space, all after are variables
                            Move (Mid(sLine, (Length(sLine)-iPos), (iPos+1))) to sVars
                            // Now parse out all variables in the line
                            Set ParseString of oTokenizer to (trim(sVars))
                            Get ItemCount of oTokenizer to iItems
                            // Go through the Tokenizer and           
                            // add to array of locals
                            // Need to do this here because we don't know where
                            // the programmer will place local variables, and 
                            // we need to update the array as they occur.  And
                            // SearchArray doesn't work for this style of array
                            Move 0 to iCounter
                            While iCounter lt iItems
                                Get TokenAt of oTokenizer iCounter to sSingleVar
                                Move (SizeOfArray(sLocalVars)) to iArrayCount
                                Move sSingleVar to sLocalVars[iArrayCount][VarName]
                                Move "0" to sLocalVars[iArrayCount][VarStatus]        // Usage indicator
                                Move sOrigMethodName to sLocalVars[iArrayCount][MethodName]
                                Move iLineCount to sLocalVars[iArrayCount][Line#]
                                Increment iCounter
                            Loop
                        End
                        Else Begin
                            // *** The rest of the method ***
                            // for each line here we look in the LocalVars array
                            // If the localvar is found before END_ then we mark
                            // it in the array, after END_ we go through the local
                            // array and look for any not marked and add to a report
                            If (trim(sLine) <> "") Begin
                                Move (SizeOfArray(sLocalVars)) to iMax
                                For iCounter from 0 to (iMax-1)
                                    If (uppercase(sLine) contains uppercase(sLocalVars[iCounter][VarName])) Begin
                                        // Mark array as found
                                        Move "1" to sLocalVars[iCounter][VarStatus]
                                    End
                                Loop
                            End
                        End
        	        Until (uppercase(sLine) contains "END_")  // End_Procedure or End_Function
        	        
                    // Check the array for non used variables
                    Move (SizeOfArray(sLocalVars)) to iMax
                    For iCounter from 0 to (iMax-1)
                        If (sLocalVars[iCounter][VarStatus] = 0) Begin 
                            Increment iFound
                            If (iOutputType = 0);
                                Showln sLocalVars[iCounter][Line#] " " sLocalVars[iCounter][MethodName] " = " sLocalVars[iCounter][VarName]
                            Else;
                                Writeln channel 2 sLocalVars[iCounter][Line#] " " sLocalVars[iCounter][MethodName] " = " sLocalVars[iCounter][VarName]
                        End
                    Loop
                    // Zero the array
                    Move (ResizeArray(sLocalVars, 0)) to sLocalVars 
        	        
        
                End // If procedure, function
            End // if not //
    	Until (SeqEof)

    	Close_Input channel 1 
    
        If (iOutputType = 1) Begin 
            If (iFound > 0) Begin 
                Writeln channel 2 "------------------------------------"
                Writeln channel 2 (String(iFound) + " unused variables found!")
                Close_Output channel 2 
                Send Info_Box "All done, look at UnusedLocals.log in the source directory."
            End
            Else Begin 
                Close_Output channel 2 
                Send Info_Box "None found, congratulations!"
            End
        End
        Else Begin 
            If (iFound = 0)Showln "None found, congratulations!"
            Else Begin 
                Showln "------------------------------------------------------"
                Showln (String(iFound) + " unused variables found!")
            End
        End
    
    End_Procedure

    Object oOpenDialog is a OpenDialog
        Set Dialog_Caption to "Select source code"
        Set Initial_Folder    to (sGetSourceDirectory())
    End_Object

    Object oSourceName is a Form
        Set Size to 13 221
        Set Location to 15 68
        Set Label_Col_Offset to 2
        Set Label_Justification_Mode to jMode_Right
        Set Label to "Source to Check:"
        Set Prompt_Button_Mode to PB_PromptOn
        Set Form_Button_Bitmap 0 To 'Prompt.Bmp'
        Set peAnchors to anTopLeftRight
        
        Procedure Prompt
            Integer bOk
            String sFileName 
            Get Show_Dialog of oOpenDialog to bOk
            If (bOk) Begin
                Get File_Name of (oOpenDialog(Self)) to sFileName
                Set Value to sFileName
            End    
        End_Procedure
    End_Object

    Object oOutputRadioGroup is a RadioGroup
        Set Location to 45 47
        Set Size to 30 201
        Set Label to "Output"
        Set peAnchors to anTopLeftRight
    
        Object oRadio1 is a Radio
            Set Label to "Output Box"
            Set Size to 10 61
            Set Location to 12 16
        End_Object
    
        Object oRadio2 is a Radio
            Set Label to "File - UnusedLocals.log"
            Set Size to 10 93
            Set Location to 12 99
        End_Object
    
        Procedure Notify_Select_State Integer iToItem Integer iFromItem
            Forward Send Notify_Select_State iToItem iFromItem
        End_Procedure
    
    End_Object

    Object oProcess_Btn is a Button
        Set Location to 100 170
        Set Label to "Process"
        Set peAnchors to anBottomRight
    
        // fires when the button is clicked
        Procedure OnClick
            Send DoParseLocals    
        End_Procedure
    
    End_Object

    Object oExit_Btn is a Button
        Set Location to 100 236
        Set Label to "Exit"
        Set peAnchors to anBottomRight
    
        Procedure OnClick
            Send Exit_Application
        End_Procedure
    
    End_Object

Cd_End_Object
